"""
Project: NSF Award Data
Author: Shubhanshu Mishra <smishra8@illinois.edy>
NSF Grant ID: 1348742
NSF Grant Title: Collaborative Research: STEM Workforce Training: A Quasi-Experimental Approach Using the Effects of Research Funding
Investigator(s): Vetle Torvik vtorvik@illinois.edu (Principal Investigator)
Date: 12/9/2013
Copyright 2013, University of Illinois at Urbana-Champaign
"""

import xmltodict, json, datetime
from ordereddict import OrderedDict
from mappingClasses import AwardDB, InvestigatorDB, ProgramElementDB, ProgramReferenceDB 

class Award:
  def __init__(self, xml_path):
    xml_fp = open(xml_path)
    jsonData = xmltodict.parse(xml_fp)
    self.data = jsonData['rootTag']['Award']
    self.table = AwardDB()
    self.investigators = [] # InvestigatorDB()
    self.program_element = [] # ProgramElementDB()
    self.program_reference = [] # ProgramReferenceDB()
    print self.data
  
  def setupInvestigator(self, investigator_data):
    investigator = InvestigatorDB()
    investigator.AwardID = self.data["AwardID"]
    investigator.RoleCode = self.getKeyData(investigator_data,"RoleCode")
    investigator.FirstName = self.getKeyData(investigator_data, "FirstName")
    investigator.LastName = self.getKeyData(investigator_data, "LastName")
    investigator.EmailAddress = self.getKeyData(investigator_data, "EmailAddress")
    investigator.StartDate = self.getDate(self.getKeyData(investigator_data, "StartDate"))
    investigator.EndDate = self.getDate(self.getKeyData(investigator_data, "EndDate"))
    return investigator

  def setupInvestigators(self):
    investigator_list = self.getKeyData(self.data,"Investigator")
    if investigator_list is None:
      return
    if type(investigator_list) is not list:
      self.investigators.append(self.setupInvestigator(investigator_list))
      return
    for i in range(len(investigator_list)):
      self.investigators.append(self.setupInvestigator(investigator_list[i]))

  def setupPECodes(self):
    PCode_list = self.getKeyData(self.data, "ProgramElement")
    if PCode_list is None:
      return
    if type(PCode_list) is not list:
      self.setupPCode(PCode_list, "PE")
      return
    for i in range(len(PCode_list)):
      self.setupPCode(PCode_list[i], "PE")

  def setupPRCodes(self):
    PRCode_list = self.getKeyData(self.data, "ProgramReference")
    if PRCode_list is None:
      return
    if type(PRCode_list) is not list:
      self.setupPCode(PRCode_list, "PR")
      return
    for i in range(len(PRCode_list)):
      self.setupPCode(PRCode_list[i], "PR")

  def setupPCode(self, pcode_data, code_type):
    if self.getKeyData(pcode_data, "Code") is None or self.getKeyString(pcode_data, "Code") is '':
      return
    p_code = ProgramReferenceDB() if code_type is "PR" else ProgramElementDB()
    print "New P Code created of type: ", type(p_code)
    p_code.Code = self.getKeyData(pcode_data, "Code")
    p_code.Text = self.getKeyData(pcode_data, "Text")
    self.program_reference.append(p_code) if code_type is "PR" else self.program_element.append(p_code)
  

  def getPECodes(self, code_list):
    if code_list is None:
      return ""
    if type(code_list) is not list:
      return code_list["Code"]
    codes = []
    for i in range(0, len(code_list)):
      codes.append(code_list[i]["Code"])
    return ",".join(codes)
  
  def getPRCodes(self, code_list):
    if code_list is None:
      return ""
    if type(code_list) is not list:
      return code_list["Code"]
    codes = []
    for i in range(0, len(code_list)):
      codes.append(self.getKeyString(code_list[i], "Code"))
    return ",".join(codes)

  def getDate(self, date_str):
    if date_str is not None:
      return datetime.datetime.strptime(date_str, "%m/%d/%Y").date()
    return None

  def getPIName(self):
    investigator_data = self.getKeyData(self.data, "Investigator")
    if investigator_data is None:
      return ""
    if type(self.data["Investigator"]) is list:
      return self.getFullName(self.data["Investigator"][0])
    return self.getFullName(self.data["Investigator"])

  def getPIEmail(self):
    investigator_data = self.getKeyData(self.data, "Investigator")
    if investigator_data is None:
      return ""
    if type(self.data["Investigator"]) is list:
      return self.data["Investigator"][0]["EmailAddress"]
    return self.data["Investigator"]["EmailAddress"]
  
  def getCoPIs(self, co_pi_dict):
    if type(co_pi_dict) is not list:
      return ""
    if len(co_pi_dict) < 2:
      return ""
    fullnames = []
    for i in range(1, len(co_pi_dict)):
      fullnames.append(self.getFullName(co_pi_dict[i]))
    return ",".join(fullnames)
  
  def getFullName(self, investigator_dict):
    return self.getKeyString(investigator_dict, "FirstName") + " " + self.getKeyString(investigator_dict,"LastName")

  def getKeyString(self, obj, key):
    if key in obj:
      if obj[key] is None:
        return ""
      return obj[key]
    return ""
  
  def getKeyData(self, obj, key):
    if key in obj:
      return obj[key]
    return None

  def sortInvestigatorList(self):
    investigator_data = self.getKeyData(self.data, "Investigator")
    if type(investigator_data) is list:
      self.data["Investigator"] = sorted(investigator_data, key=lambda x: x['RoleCode'])
      print self.data["Investigator"]
  
  def createTableObj(self):
    self.sortInvestigatorList()
    self.table.__dict__["AwardID"] = self.getKeyData(self.data, "AwardID")
    self.table.__dict__["AwardTitle"] = self.getKeyData(self.data, "AwardTitle") 
    self.table.__dict__["AwardEffectiveDate"] = self.getDate(self.getKeyData(self.data, "AwardEffectiveDate"))
    self.table.__dict__["AwardExpirationDate"] = self.getDate(self.getKeyData(self.data, "AwardExpirationDate"))
    self.table.__dict__["AwardAmount"] = self.getKeyData(self.data, "AwardAmount")
    self.table.__dict__["AwardInstrument"] = self.getKeyData(self.data["AwardInstrument"],"Value")
    self.table.__dict__["NSFOrganizationCode"] = self.getKeyData(self.data["Organization"],"Code")
    self.table.__dict__["NSFDirectorate"] = self.getKeyData(self.data["Organization"]["Directorate"],"LongName")
    self.table.__dict__["NSFDivision"] = self.getKeyData(self.data["Organization"]["Division"],"LongName")
    self.table.__dict__["ProgramOfficer"] = self.getKeyData(self.data["ProgramOfficer"],"SignBlockName")
    self.table.__dict__["AbstractNarration"] = self.getKeyData(self.data, "AbstractNarration") 
    self.table.__dict__["MinAmdLetterDate"] = self.getDate(self.getKeyData(self.data, "MinAmdLetterDate"))
    self.table.__dict__["MaxAmdLetterDate"] = self.getDate(self.getKeyData(self.data, "MaxAmdLetterDate"))
    self.table.__dict__["ARRAAmount"] = self.getKeyData(self.data, "ARRAAmount")
    self.table.__dict__["PrincipalInvestigator"] = self.getPIName()
    self.table.__dict__["PIEmailAddress"] = self.getPIEmail()
    self.table.__dict__["CoPINames"] = self.getCoPIs(self.getKeyData(self.data, "Investigator"))
    self.table.__dict__["Institution"] = self.getKeyData(self.data["Institution"], "Name")
    self.table.__dict__["InstitutionStreet"] = self.getKeyData(self.data["Institution"], "StreetAddress")
    self.table.__dict__["InstitutionCity"] = self.getKeyData(self.data["Institution"], "CityName")
    self.table.__dict__["InstitutionState"] = self.getKeyData(self.data["Institution"], "StateName")
    self.table.__dict__["InstitutionCountry"] = self.getKeyData(self.data["Institution"], "CountryName")
    self.table.__dict__["InstitutionZip"] = self.getKeyData(self.data["Institution"], "ZipCode")
    self.table.__dict__["InstitutionPhone"] = self.getKeyData(self.data["Institution"], "PhoneNumber")
    self.table.__dict__["ProgramElementCodes"] = self.getPECodes(self.getKeyData(self.data, "ProgramElement"))
    self.table.__dict__["ProgramReferenceCodes"] = self.getPRCodes(self.getKeyData(self.data, "ProgramReference"))
    ##Setup other table data for investigators, progran element and reference.
    self.setupInvestigators()
    self.setupPECodes()
    self.setupPRCodes()
    
  def showData(self):
    self.table.showData()
    for i in self.investigators:
       i.showData()
    self.program_element.showData()
    for i in self.program_reference:
      i.showData()
    
  def addToSession(self, session):
    print "In Award class session add method"
    # print "Session properties", session.__dict__
    session.merge(self.table)
    for i in self.investigators:
      session.merge(i)
    for i in self.program_element:
      session.merge(i)
    for i in self.program_reference:
      session.merge(i)
    print "All classes added."



if __name__ == "__main__":
  print "Use as module."
